import networkx as nx
import numpy as np
from sklearn.model_selection import train_test_split as tts
from sklearn.linear_model import LogisticRegression
from sklearn.metrics import classification_report as cr
from sklearn.metrics import confusion_matrix as cm
# import pickle
import config
import sys
import create_data_file


def compute_graph_stats(g):
    # Author Ke Wu, wkcoke.work@gmail.com
    stats = list()
    stats.append(nx.transitivity(g))  # fraction of all possible triangles
    stats.append(np.average(nx.clustering(g).values()))  # average clustering coeff
    stats.append(nx.average_node_connectivity(g))
    stats.append(nx.edge_connectivity(g))
    stats.append(np.average(nx.eccentricity(g).values()))  # Average eccentricity
    stats.append(nx.diameter(g))  # Max eccentricity
    stats.append(nx.radius(g))  # Min eccentricity
    stats.append(nx.average_shortest_path_length(g))
    degrees = nx.degree(g)
    stats.append(np.mean(degrees.values()))  # Average degree
    stats.append(degrees.values().count(1) / float(nx.number_of_nodes(g)))
    stats.append(np.average(nx.closeness_centrality(g).values()))  # Average path length
    stats.append(len(nx.center(g)) / float(nx.number_of_nodes(g)))  # % of central points
    stats.append(nx.density(g))  # edge density = 2m / (n*(n-1))
    stats.append(np.average(nx.average_neighbor_degree(g).values()))  # Assortativity measure.
    # Average neighbor degree, averaged over all nodes
    eig_values, eig_vectors = np.linalg.eig(nx.to_numpy_matrix(g))
    eig_values = [abs(v) for v in eig_values]
    eigs_sorted = sorted(eig_values)
    stats.append(eigs_sorted[-1])  # Largest magnitude eig value
    stats.append(eigs_sorted[-2])  # 2nd Largest magnitude eig value
    return stats


def main(nodes):
    # Create the datasets if they don't exist already
    try:
        data = np.load(config.datafile)
        labels = np.load(config.labels)
    except IOError:
        create_data_file.main()
        data = np.load(config.datafile)
        labels = np.load(config.labels)
    if data.shape[1] != nodes:
        print('Subgraph size does not match data file. Terminating...')
        exit(1)
    try:
        x = np.load(config.statfile)
    except IOError:
        print('Computing classical features...')
        x = np.empty([data.shape[0], 16])
        for i in range(data.shape[0]):
            graph = nx.from_numpy_matrix(data[i, :, :])
            x[i, :] = compute_graph_stats(graph)
        np.save(config.statfile, x)

    x_train, x_test, y_train, y_test = tts(x, labels, test_size=0.5)
    model = LogisticRegression()
    if len(np.unique(labels)) < 2:
        print('You have only one class in the data. We need at least 2. Terminating...')
        exit(1)
    model.fit(x_train, y_train)

    # Save model
    # filename = 'models/lr_' + nodes + '.pkl'
    # pickle.dump(model, open(filename, 'wb'))

    predicts = model.predict(x_test)
    with open('results/lr_' + str(nodes) + '.txt', 'w') as f:
        f.write('Data file: ' + config.datafile + '\n')
        f.write(cr(y_test, predicts, target_names=config.target))
        f.write('\n')
        f.write(cm(y_test, predicts))
    # print 'LR: ' + nodes
    # print cr(y_test, predicts, target_names=target)
    # print cm(y_test, predicts)


if __name__ == '__main__':
    main(sys.argv[1])
