import numpy as np
from keras.models import Sequential
from keras.layers import Dense
from keras.layers import Dropout
from keras.layers import Flatten
from keras.layers.convolutional import Conv2D
from keras.layers.convolutional import MaxPooling2D
from keras.utils import np_utils
from keras import backend as k
from sklearn.metrics import classification_report, confusion_matrix
import sys
import config
import create_data_file


def create_model(num_nodes, num_classes):
    model = Sequential()
    model.add(Conv2D(32, (5, 5), padding='valid', input_shape=(1, num_nodes, num_nodes), activation='relu'))
    model.add(MaxPooling2D(pool_size=(2, 2)))
    model.add(Dropout(0.2))
    model.add(Flatten())
    model.add(Dense(128, activation='relu'))
    model.add(Dense(num_classes, activation='softmax'))
    model.compile(loss='categorical_crossentropy', optimizer='adam', metrics=['accuracy'])
    return model


def main(nodes):
    k.set_image_dim_ordering('th')
    np.random.seed(7)
    # Create the datasets if they don't exist already
    try:
        x = np.load(config.datafile)
        y = np.load(config.labels)
    except IOError:
        create_data_file.main()
        x = np.load(config.datafile)
        y = np.load(config.labels)
    if x.shape[1] != nodes:
        print('Subgraph size does not match data file. Terminating...')
        exit(1)
    # y -= 1  # if the labeling starts from 1. it must start from 0
    n_samples = x.shape[0]
    perm = np.random.permutation(x.shape[0])
    x = np.take(x, perm, axis=0)
    y = np.take(y, perm)
    y = np_utils.to_categorical(y).astype('int32')

    x_train = x[:n_samples // 3, :, :]
    x_valid = x[n_samples // 3:(2 * n_samples) // 3, :, :]
    x_test = x[(2 * n_samples) // 3:, :, :]

    y_train = y[:n_samples // 3, :]
    y_valid = y[n_samples // 3:(2 * n_samples) // 3, :]
    y_test = y[(2 * n_samples) // 3:, :]
    actuals = y_test.argmax(1).astype('int32')

    # Reshape data as required by Keras
    nodes = int(nodes)
    x_train = x_train.reshape(x_train.shape[0], 1, nodes, nodes).astype('float32')
    x_valid = x_valid.reshape(x_valid.shape[0], 1, nodes, nodes).astype('float32')
    x_test = x_test.reshape(x_test.shape[0], 1, nodes, nodes).astype('float32')

    # Model creation
    num_classes = y_test.shape[1]
    if num_classes < 2:
        print('You have only one class in the data. We need at least 2. Terminating...')
        exit(1)
    modelo = create_model(nodes, num_classes)

    # Model training
    print('Model is training...')
    modelo.fit(x_train, y_train, validation_data=(x_valid, y_valid), epochs=100, batch_size=200, verbose=2)

    # Save the model
    model_json = modelo.to_json()
    with open('models/cnn_model' + str(nodes) + '.json', "w") as json_file:
        json_file.write(model_json)
    modelo.save_weights('models/cnn_model' + str(nodes) + '.h5')
    print("Saved model to disk")

    # Model evaluation
    preds = modelo.predict(x_test)
    preds = preds.argmax(1).astype('int32')
    with open('results/cnn_' + str(nodes) + '.txt', 'w') as f:
        f.write('Data file: ' + config.datafile + '\n')
        f.write(classification_report(actuals, preds, target_names=config.target))
        f.write('\n')
        f.write(confusion_matrix(actuals, preds))
    # print 'NN:' + str(nodes)
    # print classification_report(actuals, preds, target_names=target)
    # print confusion_matrix(actuals, preds)


if __name__ == '__main__':
    main(sys.argv[1])
